// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:go_router/go_router.dart';
// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';

typedef ThemedRoute = GoRoute;
typedef ThemedShellRoute = ShellRoute;
typedef ThemedRouteBase = RouteBase;
typedef ThemedRouteState = GoRouterState;

typedef ThemedRouter = GoRouter;

class ThemedApp extends StatelessWidget {
  final String title;
  final bool debugShowCheckedModeBanner;
  final bool showPerformanceOverlay;
  final Widget errorPageWidget;
  final List<ThemedRouteBase> routes;
  final List<NavigatorObserver>? observers;
  final Widget Function(
    BuildContext context,
    ThemedRouteState state,
    Widget child,
  )? pageBuilder;
  final GlobalKey<NavigatorState>? rootNavigatorKey;
  final GlobalKey<NavigatorState>? nestedNavigatorKey;
  final String initialLocation;
  final Future<String?> Function(BuildContext, ThemedRouteState)? redirect;

  const ThemedApp({
    Key? key,
    required this.title,
    required this.routes,
    this.pageBuilder,
    this.initialLocation = '/',
    this.rootNavigatorKey,
    this.nestedNavigatorKey,
    this.errorPageWidget = const ThemedMessage(
      message: "404 - Unknown Page",
    ),
    this.observers,
    this.debugShowCheckedModeBanner = true,
    this.showPerformanceOverlay = false,
    this.redirect,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ProviderScope(
      child: _ThemedAppCore(
        title: title,
        routes: routes,
        debugShowCheckedModeBanner: debugShowCheckedModeBanner,
        errorPageWidget: errorPageWidget,
        initialLocation: initialLocation,
        key: key,
        nestedNavigatorKey: nestedNavigatorKey,
        observers: observers,
        pageBuilder: pageBuilder,
        rootNavigatorKey: rootNavigatorKey,
        showPerformanceOverlay: showPerformanceOverlay,
        redirect: redirect,
      ),
    );
  }
}

class _ThemedAppCore extends ConsumerWidget {
  final String title;
  final bool debugShowCheckedModeBanner;
  final bool showPerformanceOverlay;
  final Widget errorPageWidget;
  final List<ThemedRouteBase> routes;
  final List<NavigatorObserver>? observers;
  final Widget Function(
    BuildContext context,
    ThemedRouteState state,
    Widget child,
  )? pageBuilder;
  final GlobalKey<NavigatorState>? rootNavigatorKey;
  final GlobalKey<NavigatorState>? nestedNavigatorKey;
  final String initialLocation;
  final Future<String?> Function(BuildContext, ThemedRouteState)? redirect;

  const _ThemedAppCore({
    Key? key,
    required this.title,
    required this.routes,
    this.pageBuilder,
    this.initialLocation = '/',
    this.rootNavigatorKey,
    this.nestedNavigatorKey,
    this.errorPageWidget = const ThemedMessage(
      message: "404 - Unknown Page",
    ),
    this.observers,
    this.debugShowCheckedModeBanner = true,
    this.showPerformanceOverlay = false,
    this.redirect,
  }) : super(key: key);

  GoRouter get _router => GoRouter(
        navigatorKey: rootNavigatorKey,
        initialLocation: initialLocation,
        redirect: redirect,
        routes: routes,
        observers: observers,
        errorPageBuilder: (context, state) => MaterialPage<void>(
          key: state.pageKey,
          child: errorPageWidget,
        ),
      );

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    return MaterialApp.router(
      title: title,
      locale:
          themedLocalToLocale[ref.watch(vertticeUIPreferenceProvider).locale],
      localizationsDelegates: const [
        GlobalMaterialLocalizations.delegate,
        GlobalWidgetsLocalizations.delegate,
        GlobalCupertinoLocalizations.delegate,
        GlobalWidgetsLocalizations.delegate,
      ],
      supportedLocales: VertticeUI.instance.localeManager.appSupportLocale
          .map((e) => themedLocalToLocale[e]!)
          .toList(),
      routerDelegate: _router.routerDelegate,
      routeInformationParser: _router.routeInformationParser,
      routeInformationProvider: _router.routeInformationProvider,
      color: ref.watch(vertticeUIPreferenceProvider).themedColors.accentColor,
      theme: ThemeData(
        useMaterial3: true,
        primaryColor:
            ref.watch(vertticeUIPreferenceProvider).themedColors.accentColor,
        colorScheme: ColorScheme.fromSeed(
          seedColor:
              ref.watch(vertticeUIPreferenceProvider).themedColors.accentColor,
          brightness: Brightness.light,
        ),
      ),
      darkTheme: ThemeData(
        useMaterial3: true,
        primaryColor:
            ref.watch(vertticeUIPreferenceProvider).themedColors.accentColor,
        colorScheme: ColorScheme.fromSeed(
          seedColor:
              ref.watch(vertticeUIPreferenceProvider).themedColors.accentColor,
          brightness: Brightness.dark,
        ),
      ),
      themeMode: ref.watch(vertticeUIPreferenceProvider).isDarkTheme
          ? ThemeMode.dark
          : ThemeMode.light,
      showPerformanceOverlay: showPerformanceOverlay,
      debugShowCheckedModeBanner: debugShowCheckedModeBanner,
    );
    //   },
    // );
  }
}
