// 🎯 Dart imports:
import "dart:async";

// 🐦 Flutter imports:
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:cross_connectivity/cross_connectivity.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:verttice_ui/verttice_ui.dart';

import 'package:wakelock/wakelock.dart';

class ThemedColors {
  final Color textColor;
  final Color accentColor;
  final String accentColorName;

  const ThemedColors({
    required this.textColor,
    required this.accentColor,
    required this.accentColorName,
  });
}

final vertticeUIPreferenceProvider =
    StateNotifierProvider<VertticeUIPreferenceNotifier, VertticeUIPreference>(
        (ref) {
  return VertticeUIPreferenceNotifier();
});

@immutable
class VertticeUIPreference {
  final ThemedColors themedColors;
  final bool isDarkTheme;
  final ThemedLocale locale;

  const VertticeUIPreference({
    this.themedColors = const ThemedColors(
      accentColor: Colors.red,
      textColor: Colors.white,
      accentColorName: 'red',
    ),
    this.isDarkTheme = false,
    this.locale = ThemedLocale.en,
  });

  VertticeUIPreference copyWith({
    ThemedColors? themedColors,
    bool? isDarkTheme,
    ThemedLocale? locale,
  }) {
    return VertticeUIPreference(
      themedColors: themedColors ?? this.themedColors,
      isDarkTheme: isDarkTheme ?? this.isDarkTheme,
      locale: locale ?? this.locale,
    );
  }
}

class VertticeUIPreferenceNotifier extends StateNotifier<VertticeUIPreference> {
  VertticeUIPreferenceNotifier() : super(const VertticeUIPreference()) {
    _initTheme();
    _initLocale();
  }

  Future _initTheme() async {
    final pref = await SharedPreferences.getInstance();
    final dark = pref.getBool("dark") ?? false;
    final themedColors = pref.getString("themedColors") ?? 'red';
    await setThemedColors(themedColors);
    await setBrightnessTheme(dark);
  }

  Future<void> setThemedColors(String color) async {
    final pref = await SharedPreferences.getInstance();
    await pref.setString("themedColors", color);
    _colorSwitch(color);

    state = state.copyWith(
      themedColors: _colorSwitch(color),
    );
  }

  Future<void> setBrightnessTheme(bool isDark) async {
    final pref = await SharedPreferences.getInstance();
    await pref.setBool("dark", isDark);
    state = state.copyWith(
      isDarkTheme: isDark,
    );
  }

  void setLocale(ThemedLocale locale) {
    state = state.copyWith(
      locale: locale,
    );

    VertticeUI.instance.localeManager.initLocale(locale);
  }

  void _initLocale() {
    state = state.copyWith(
      locale: VertticeUI.instance.localeManager.locale,
    );
  }

  ThemedColors _colorSwitch(String color) {
    switch (color) {
      case 'orange':
        return const ThemedColors(
          accentColor: Colors.orange,
          textColor: Colors.white,
          accentColorName: 'orange',
        );

      case 'cyan':
        return const ThemedColors(
          accentColor: Colors.cyan,
          textColor: Colors.black,
          accentColorName: 'cyan',
        );

      case 'purple':
        return const ThemedColors(
          accentColor: Colors.purple,
          textColor: Colors.white,
          accentColorName: 'purple',
        );
      case 'blue':
        return const ThemedColors(
          accentColor: Colors.blue,
          textColor: Colors.black,
          accentColorName: 'blue',
        );
      case 'teal':
        return const ThemedColors(
          accentColor: Colors.teal,
          textColor: Colors.black,
          accentColorName: 'teal',
        );
      case 'green':
        return const ThemedColors(
          accentColor: Colors.green,
          textColor: Colors.white,
          accentColorName: 'green',
        );

      default:
        return const ThemedColors(
          accentColor: Colors.red,
          textColor: Colors.white,
          accentColorName: 'red',
        );
    }
  }
}
